/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package javagi.eclipse.jdt.internal.compiler.parser;

/**
 * IMPORTANT NOTE: These constants are dedicated to the internal Scanner implementation. 
 * It is mirrored in javagi.eclipse.jdt.core.compiler public package where it is API. 
 * The mirror implementation is using the backward compatible ITerminalSymbols constant 
 * definitions (stable with 2.0), whereas the internal implementation uses TerminalTokens 
 * which constant values reflect the latest parser generation state.
 */
/**
 * Maps each terminal symbol in the java-grammar into a unique integer. 
 * This integer is used to represent the terminal when computing a parsing action. 
 * 
 * Disclaimer : These constant values are generated automatically using a Java 
 * grammar, therefore their actual values are subject to change if new keywords 
 * were added to the language (for instance, 'assert' is a keyword in 1.4).
 */
public interface TerminalTokens {

	// special tokens not part of grammar - not autogenerated
	int TokenNameWHITESPACE = 1000,
		TokenNameCOMMENT_LINE = 1001,
		TokenNameCOMMENT_BLOCK = 1002,
		TokenNameCOMMENT_JAVADOC = 1003;

//*BEGIN_INPUT* javasym.java
    public final static int
      TokenNameIdentifier = 26,
      TokenNameabstract = 57,
      TokenNameassert = 78,
      TokenNameboolean = 32,
      TokenNamebreak = 79,
      TokenNamebyte = 33,
      TokenNamecase = 106,
      TokenNamecatch = 108,
      TokenNamechar = 34,
      TokenNameclass = 73,
      TokenNamecontinue = 80,
      TokenNameconst = 113,
      TokenNamedefault = 103,
      TokenNamedo = 81,
      TokenNamedouble = 35,
      TokenNameelse = 109,
      TokenNameenum = 100,
      TokenNameextends = 82,
      TokenNamefalse = 46,
      TokenNamefinal = 58,
      TokenNamefinally = 110,
      TokenNamefloat = 36,
      TokenNamefor = 83,
      TokenNamegoto = 114,
      TokenNameif = 84,
      TokenNameimplements = 101,
      TokenNameimport = 104,
      TokenNameinstanceof = 14,
      TokenNameint = 37,
      TokenNameinterface = 76,
      TokenNamelong = 38,
      TokenNamenative = 59,
      TokenNamenew = 43,
      TokenNamenull = 47,
      TokenNamepackage = 102,
      TokenNameprivate = 60,
      TokenNameprotected = 61,
      TokenNamepublic = 62,
      TokenNamereturn = 85,
      TokenNameshort = 39,
      TokenNamestatic = 48,
      TokenNamestrictfp = 63,
      TokenNamesuper = 41,
      TokenNameswitch = 86,
      TokenNamesynchronized = 49,
      TokenNamethis = 42,
      TokenNamethrow = 87,
      TokenNamethrows = 111,
      TokenNametransient = 64,
      TokenNametrue = 50,
      TokenNametry = 88,
      TokenNamevoid = 40,
      TokenNamevolatile = 65,
      TokenNamewhile = 77,
      TokenNamewhere = 105,
      TokenNamereceiver = 107,
      TokenNameimplementation = 45,
      TokenNameIntegerLiteral = 51,
      TokenNameLongLiteral = 52,
      TokenNameFloatingPointLiteral = 53,
      TokenNameDoubleLiteral = 54,
      TokenNameCharacterLiteral = 55,
      TokenNameStringLiteral = 56,
      TokenNamePLUS_PLUS = 8,
      TokenNameMINUS_MINUS = 9,
      TokenNameEQUAL_EQUAL = 18,
      TokenNameLESS_EQUAL = 15,
      TokenNameGREATER_EQUAL = 16,
      TokenNameNOT_EQUAL = 19,
      TokenNameLEFT_SHIFT = 17,
      TokenNameRIGHT_SHIFT = 10,
      TokenNameUNSIGNED_RIGHT_SHIFT = 11,
      TokenNamePLUS_EQUAL = 89,
      TokenNameMINUS_EQUAL = 90,
      TokenNameMULTIPLY_EQUAL = 91,
      TokenNameDIVIDE_EQUAL = 92,
      TokenNameAND_EQUAL = 93,
      TokenNameOR_EQUAL = 94,
      TokenNameXOR_EQUAL = 95,
      TokenNameREMAINDER_EQUAL = 96,
      TokenNameLEFT_SHIFT_EQUAL = 97,
      TokenNameRIGHT_SHIFT_EQUAL = 98,
      TokenNameUNSIGNED_RIGHT_SHIFT_EQUAL = 99,
      TokenNameOR_OR = 25,
      TokenNameAND_AND = 24,
      TokenNamePLUS = 1,
      TokenNameMINUS = 2,
      TokenNameNOT = 68,
      TokenNameREMAINDER = 5,
      TokenNameXOR = 21,
      TokenNameAND = 20,
      TokenNameMULTIPLY = 4,
      TokenNameOR = 22,
      TokenNameTWIDDLE = 69,
      TokenNameDIVIDE = 6,
      TokenNameGREATER = 12,
      TokenNameLESS = 7,
      TokenNameLPAREN = 27,
      TokenNameRPAREN = 29,
      TokenNameLBRACE = 70,
      TokenNameRBRACE = 31,
      TokenNameLBRACKET = 13,
      TokenNameRBRACKET = 67,
      TokenNameSEMICOLON = 28,
      TokenNameQUESTION = 23,
      TokenNameCOLON = 66,
      TokenNameCOLONCOLON = 72,
      TokenNameHASH = 75,
      TokenNameCOMMA = 30,
      TokenNameDOT = 3,
      TokenNameEQUAL = 74,
      TokenNameAT = 44,
      TokenNameELLIPSIS = 112,
      TokenName$eof = 71,
      TokenName$error = 115;
//*END_INPUT* javasym.java
    
    public final static int
    TokenNameEOF = TokenName$eof,
    TokenNameERROR = TokenName$error;
}
